/* fgkicker - Flightgear launcher

   Copyright (C) 2003 Visionary Software Developments
   Written by Chris De Maeyer <chrisdm@pandora.be>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
*
* $Author: chris $
* $Revision: 1.8 $
* $Log: fgkfile.c,v $
* Revision 1.8  2004/06/26 18:07:28  chris
* * airport list is searchable
* * changed code to use gtk types
* * added tooltips
* * gtk_widget_show replaced by gtk_widget_show_all
* * added "FgKicker" page, added "disable tooltips" option
*
* Revision 1.7  2004/06/03 08:34:41  chris
* * added "enable httpd host" option
* * fixed strsize issue handling aircraft list (Joseph Harnish <bigjoe@linuxmail.org>)
* * fixed writing out settings file for horizon/lighting option
*
* Revision 1.6  2004/03/18 19:53:44  chris
* aircraft list now gets read from fgfs
*
* Revision 1.5  2004/01/11 15:10:22  chris
* * added tree view for airport selection
* * fixed info window
* * data files now read from user's HOME directory
*
* Revision 1.4  2003/12/14 19:26:33  chris
* * re-arranged code on a per page topic base
* * added notebook with pages
* * added options "time match real", "skyblend", "textures", "clouds", "units"
* * added info button
*
* Revision 1.3  2003/12/09 21:22:15  chris
* * argument build now gets cleared properly between fg invocations
* * using fgfs iso /usr/games/fgfs to start, should work as long as the binary is in
*   your search path
* * added error messages
* * added options random-objects, freeze
* * aircraft list read from text file generated by "fgfs --show-aircraft > aircraft.dat",
*   file should be in your HOME
*
* Revision 1.2  2003/12/04 19:32:34  chris
* added load/save default settings
*
* Revision 1.1  2003/11/29 15:12:25  chris
* initial version
*
*/

#include "fgkicker.h"

extern GList *combo_aircraft_list;
extern GList *combo_airports_list;

gint fgk_SettingsThere(gchar *file)
{
	FILE *tmp;

	if((tmp=fopen(file,"r"))!=NULL) {
		fclose(tmp);
		return(0);
	} else {
		return(-1);
	}
}	

gint fgk_GetSettings(SETTINGS *current,gchar *file)
{
	FILE *tmp;
	gchar str[256];
	gchar *ipos;
	gchar token[64];
	gchar x[2];
	gint tag;

	if((tmp=fopen(file,"r"))!=NULL) {
		while(!feof(tmp)) {
			fgets(str,sizeof(str)-1,tmp);

			ipos=strchr(str,'=');

			if (ipos!=NULL) {
				strncpy(token,str,ipos-str);
				token[ipos-str]=0;

				tag=0;
				if(strcmp(token,"intromusic")==0) {
					tag=1;
				} else if (strcmp(token,"splashscreen")==0) {
					tag=2;
				} else if (strcmp(token,"gamemode")==0) {
					tag=3;
				} else if (strcmp(token,"fuelmode")==0) {
					tag=4;
				} else if (strcmp(token,"mousepointer")==0) {
					tag=5;
				} else if (strcmp(token,"clockfreeze")==0) {
					tag=6;
				} else if (strcmp(token,"autocoordination")==0) {
					tag=7;
				} else if (strcmp(token,"randomobjects")==0) {
					tag=8;
				} else if (strcmp(token,"freeze")==0) {
					tag=9;
				} else if (strcmp(token,"matchreal")==0) {
					tag=10;
				} else if (strcmp(token,"skyblend")==0) {
					tag=11;
				} else if (strcmp(token,"textures")==0) {
					tag=12;
				} else if (strcmp(token,"clouds")==0) {
					tag=13;
				} else if (strcmp(token,"units")==0) {
					tag=14;
				} else if (strcmp(token,"horizon")==0) {
					tag=15;
				} else if (strcmp(token,"lighting")==0) {
					tag=16;
				} else if (strcmp(token,"httpd")==0) {
					tag=17;
				}else if (strcmp(token,"tooltips")==0) {
					tag=18;
				}	
				if(tag>0) {
					strncpy(x,&str[ipos-str+1],1);
					x[1]=0;
				}
				switch(tag) {
					case 1:
						current->intromusic = atoi(x);
						break;
					case 2:
						current->splash = atoi(x);
						break;
					case 3:
						current->gamemode = atoi(x);
						break;
					case 4:
						current->fuelmode = atoi(x);
						break;
					case 5:
						current->mousepointer = atoi(x);
						break;
					case 6:
						current->clockfreeze = atoi(x);
						break;
					case 7:
						current->autocoord = atoi(x);
						break;
					case 8:
						current->robjects = atoi(x);
						break;	
					case 9:
						current->freeze = atoi(x);
						break;	
					case 10:
						current->matchreal = atoi(x);
						break;	
					case 11:
						current->skyblend = atoi(x);
						break;							
					case 12:
						current->textures = atoi(x);
						break;			
					case 13:
						current->clouds = atoi(x);
						break;
					case 14:
						current->units = atoi(x);
						break;
					case 15:
						current->horizon = atoi(x);
						break;	
					case 16:
						current->lighting = atoi(x);
						break;
					case 17:
						current->httpd = atoi(x);
						break;
					case 18:
						current->tooltips = atoi(x);
						break;		
				}
			}	
			//printf("%s\n",str);
		}
		fclose(tmp);
	} else {
		return(-1);
	}	
	return(0);
}

gint fgk_PutSettings(SETTINGS *current, gchar *file)
{
	FILE *tmp;

	if((tmp=fopen(file,"w"))!=NULL) {
		fprintf(tmp,"intromusic=%d\n",current->intromusic);
		fprintf(tmp,"splashscreen=%d\n",current->splash);
		fprintf(tmp,"gamemode=%d\n",current->gamemode);
		fprintf(tmp,"fuelmode=%d\n",current->fuelmode);
		fprintf(tmp,"mousepointer=%d\n",current->mousepointer);
		fprintf(tmp,"clockfreeze=%d\n",current->clockfreeze);
		fprintf(tmp,"autocoordination=%d\n",current->autocoord);
		fprintf(tmp,"randomobjects=%d\n",current->robjects);
		fprintf(tmp,"freeze=%d\n",current->freeze);
		fprintf(tmp,"matchreal=%d\n",current->matchreal);
		fprintf(tmp,"skyblend=%d\n",current->skyblend);
		fprintf(tmp,"textures=%d\n",current->textures);
		fprintf(tmp,"clouds=%d\n",current->clouds);
		fprintf(tmp,"units=%d\n",current->units);
		fprintf(tmp,"horizon=%d\n",current->horizon);
		fprintf(tmp,"lighting=%d\n",current->lighting);
		fprintf(tmp,"httpd=%d\n",current->httpd);
		fprintf(tmp,"tooltips=%d\n",current->tooltips);
		fprintf(tmp,"airport=%s\n",current->airport);
		fprintf(tmp,"aircraft=%s\n",current->aircraft);
		fclose(tmp);
	} else {
		return(-1);
	}	
	return(0);	
}

gint fgk_ReadAirports(gchar *flyfile,GtkTreeStore *st)
{
	FILE *tmp;
	gchar ffile[160];
	gchar str[256];
	gchar word2[32];
	gchar word7[32];
	GtkTreeIter iter1;
	GString *tok2,*tok7;

	sprintf(ffile, "%s/%s", getenv("HOME"), flyfile);

	if((tmp=fopen(ffile,"r"))!=NULL) {
		while(!feof(tmp)) {
			fgets(str,sizeof(str)-1,tmp);
			str[strlen(str)-1]=0; /* remove newline */

			memset(word2,0,32);
			strncpy(word2,str,4);

			memset(word7,0,32);
			strncpy(word7,str+5,32);

			tok2 = g_string_new(word2);
			tok7 = g_string_new(word7);

			gtk_tree_store_append (st, &iter1, NULL);  /* Acquire a top-level iterator */
			gtk_tree_store_set (st, &iter1,
				NAME_COLUMN, tok7->str,
				ID_COLUMN, tok2->str,
				-1);
		}
		fclose(tmp);
	} else {
		return(-1);
	}	
	return(0);	
}	
