/* fgkicker - Flightgear launcher

   Copyright (C) 2003 Visionary Software Developments
   Written by Chris De Maeyer <chrisdm@pandora.be>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
*
* $Author: chris $
* $Revision: 1.8 $
* $Log: fgkicker.c,v $
* Revision 1.8  2004/06/26 18:07:29  chris
* * airport list is searchable
* * changed code to use gtk types
* * added tooltips
* * gtk_widget_show replaced by gtk_widget_show_all
* * added "FgKicker" page, added "disable tooltips" option
*
* Revision 1.7  2004/06/03 08:34:42  chris
* * added "enable httpd host" option
* * fixed strsize issue handling aircraft list (Joseph Harnish <bigjoe@linuxmail.org>)
* * fixed writing out settings file for horizon/lighting option
*
* Revision 1.6  2004/03/18 19:53:55  chris
* aircraft list now gets read from fgfs
*
* Revision 1.5  2004/01/11 15:10:23  chris
* * added tree view for airport selection
* * fixed info window
* * data files now read from user's HOME directory
*
* Revision 1.4  2003/12/14 19:26:34  chris
* * re-arranged code on a per page topic base
* * added notebook with pages
* * added options "time match real", "skyblend", "textures", "clouds", "units"
* * added info button
*
* Revision 1.3  2003/12/09 21:22:16  chris
* * argument build now gets cleared properly between fg invocations
* * using fgfs iso /usr/games/fgfs to start, should work as long as the binary is in
*   your search path
* * added error messages
* * added options random-objects, freeze
* * aircraft list read from text file generated by "fgfs --show-aircraft > aircraft.dat",
*   file should be in your HOME
*
* Revision 1.2  2003/12/04 19:32:35  chris
* added load/save default settings
*
* Revision 1.1  2003/11/29 15:12:29  chris
* initial version
*
*/

/* Please read TO DO list and consider contributing */

#include "fgkicker.h"
#ifndef FGKICKER_PROTO_H
#include "fgkicker_proto.h"
#endif

GtkWidget 	*window;
GList 		*combo_aircraft_list=NULL;
gchar 		cArgumentString[1024];
SETTINGS 	current = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"",""};

GtkWidget *label_aircraft;
GtkWidget *label_airport;
GtkWidget *combo_aircraft;
GtkWidget *check_splash;
GtkWidget *check_gamemode;
GtkWidget *check_fuelmode;
GtkWidget *check_intromusic;
GtkWidget *check_mousepointer;
GtkWidget *check_clockfreeze;
GtkWidget *check_autocoord;
GtkWidget *check_robjects;
GtkWidget *check_freeze;
GtkWidget *check_matchreal;
GtkWidget *check_skyblend;
GtkWidget *check_textures;
GtkWidget *check_clouds;
GtkWidget *check_units;
GtkWidget *check_horizon;
GtkWidget *check_lighting;
GtkWidget *check_httpd;
GtkWidget *tooltips;
GtkWidget *button_quit;
GtkWidget *button_kick;
GtkWidget *button_save;
GtkWidget *button_load;
GtkWidget *button_info;

GtkTooltips *tips;

GtkTreeStore *store;
GtkWidget *tree;
GtkTreeViewColumn *column;
GtkCellRenderer *renderer;
GtkTreeSelection *sel;

GtkWidget *vboxm;
GtkWidget *hboxm;
GtkWidget *note;
GtkWidget *labelg;
GtkWidget *vboxg0;
GtkWidget 	*hboxg1;
GtkWidget		*vboxg10;
GtkWidget		*vboxg11;
GtkWidget 	*hboxg2;
GtkWidget *frameg1;
GtkWidget *frameg2;

GtkWidget *labelr;
GtkWidget *vboxr0;
GtkWidget   *hboxr1;
GtkWidget		*vboxr10;
GtkWidget		*vboxr11;
GtkWidget *framer1;

GtkWidget *labelt;
GtkWidget *vboxt0;
GtkWidget   *hboxt1;
GtkWidget		*vboxt10;
GtkWidget		*vboxt11;
GtkWidget *framet1;

GtkWidget *labelf;
GtkWidget *vboxf0;
GtkWidget   *hboxf1;
GtkWidget		*vboxf10;
GtkWidget		*vboxf11;
GtkWidget *framef1;

GtkWidget *scrolled_window;
GtkWidget *labela;
GtkWidget *vboxa0;
GtkWidget 	*vboxa1;
GtkWidget   	*hboxa0;
GtkWidget *framea1;

/* funcs */
void FillAircrafts(void) 
{
	gint res;
	gint res2;
	gchar str[128];
	gchar *ipos;
	gchar token[32];
	GString *tok;
	FILE *pd;
	gint i;
	gchar *str2;

	/* Generate Aircraft File */
	if((pd = popen("fgfs --show-aircraft", "r")) == NULL) {
		perror("Can not get aircraft list");
		strcpy(str, "unknown");
	}
	while(fgets(str, sizeof(str),pd) != NULL){
		// Pull out the first 2 spaces
		str2 = str;
		while (isspace(*str2))
			str2++;
		ipos=strchr(str2,' ');
		if(ipos!=NULL){
			strncpy(token,str2,ipos-str);
			token[ipos-str]=0;
			tok = g_string_new(token);
			combo_aircraft_list=g_list_append (combo_aircraft_list, tok->str);
		}
	}
	pclose(pd);	
}	

void FillAirports(GtkTreeStore *st) 
{
	gint res;

	/* read from a file */
	res=fgk_ReadAirports(AIRFILE,st);
	if(res<0) {
		error_message(window,"Could not read airport data file!");
	}	
}	

void BuildArguments(void)
{
	const gchar *ing;

	strcpy(cArgumentString,"fgfs");

	(current.splash == 1)?strcat(cArgumentString," --enable-splash-screen"):strcat(cArgumentString," --disable-splash-screen");
	(current.gamemode == 1)?strcat(cArgumentString," --enable-game-mode"):strcat(cArgumentString," --disable-game-mode");
	(current.fuelmode == 1)?strcat(cArgumentString," --enable-fuel-freeze"):strcat(cArgumentString," --disable-fuel-freeze");
	(current.intromusic == 1)?strcat(cArgumentString," --enable-intro-music"):strcat(cArgumentString," --disable-intro-music");
	(current.mousepointer == 1)?strcat(cArgumentString," --enable-mouse-pointer"):strcat(cArgumentString," --disable-mouse-pointer");
	(current.clockfreeze == 1)?strcat(cArgumentString," --enable-clock-freeze"):strcat(cArgumentString," --disable-clock-freeze");
	(current.fuelmode == 1)?strcat(cArgumentString," --enable-auto-coordination"):strcat(cArgumentString," --disable-auto-coordination");
	(current.robjects == 1)?strcat(cArgumentString," --enable-random-objects"):strcat(cArgumentString," --disable-random-objects");
	(current.freeze == 1)?strcat(cArgumentString," --enable-freeze"):strcat(cArgumentString," --disable-freeze");
	(current.matchreal == 1)?strcat(cArgumentString," --time-match-real"):strcat(cArgumentString," --time-match-local");
	(current.skyblend == 1)?strcat(cArgumentString," --enable-skyblend"):strcat(cArgumentString," --disable-skyblend");
	(current.textures == 1)?strcat(cArgumentString," --enable-textures"):strcat(cArgumentString," --disable-textures");
	(current.clouds == 1)?strcat(cArgumentString," --enable-clouds"):strcat(cArgumentString," --disable-clouds");
	(current.units == 1)?strcat(cArgumentString," --units-meters"):strcat(cArgumentString," --units-feet");
	(current.horizon == 1)?strcat(cArgumentString," --enable-horizon-effect"):strcat(cArgumentString," --disable-horizon-effect");
	(current.lighting == 1)?strcat(cArgumentString," --enable-enhanced-lighting"):strcat(cArgumentString," --disable-enhanced-lighting");
	if (current.httpd == 1) {
		strcat(cArgumentString," --httpd=5500");
	}	

	ing = NULL;
	ing = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_aircraft)->entry));
	strncpy(current.aircraft,ing,strlen(ing));
	strncat(cArgumentString," --aircraft=",12);
	strcat(cArgumentString,ing);

	if (strlen(current.airport)>0) {
		strncat(cArgumentString," --airport=",11);
		strcat(cArgumentString,current.airport);		
	}
}

void LoadFile(gchar *file) {
	if(fgk_GetSettings(&current,file)<0) {
		error_message(window, "No default settings file, set, then Save As Default: "OPTFILE );
	} else {
		(current.intromusic == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_intromusic),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_intromusic),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_intromusic), NULL);

		(current.splash == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_splash),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_splash),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_splash), NULL);

		(current.gamemode == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_gamemode),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_gamemode),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_gamemode), NULL);

		(current.fuelmode == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_fuelmode),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_fuelmode),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_fuelmode), NULL);

		(current.mousepointer == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_mousepointer),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_mousepointer),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_mousepointer), NULL);

		(current.clockfreeze == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_clockfreeze),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_clockfreeze),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_clockfreeze), NULL);

		(current.autocoord == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_autocoord),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_autocoord),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_autocoord), NULL);

		(current.robjects == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_robjects),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_robjects),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_robjects), NULL);

		(current.freeze == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_freeze),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_freeze),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_freeze), NULL);

		(current.matchreal == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_matchreal),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_matchreal),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_matchreal), NULL);

		(current.skyblend == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_skyblend),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_skyblend),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_skyblend), NULL);

		(current.textures == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_textures),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_textures),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_textures), NULL);

		(current.clouds == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_clouds),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_clouds),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_textures), NULL);

		(current.units == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_units),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_units),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_units), NULL);

		(current.horizon == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_horizon),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_horizon),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_horizon), NULL);

		(current.lighting == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_lighting),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_lighting),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_lighting), NULL);

		(current.httpd == 1)?gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_httpd),TRUE):gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_httpd),FALSE);
		gtk_widget_draw(GTK_WIDGET(check_httpd), NULL);

		if (current.tooltips == 1) {
			gtk_tooltips_disable(tips);
		}	

	}
}

gint Initialize(void)
{
	/* Here should come some pre-check for path availability etc */

	return(1);
}

void ErrorExit(void)
{
}	

void CleanExit(void)
{
}

static void but_quit_clicked( GtkWidget *widget, gpointer   data )
{
}

static void but_kick_clicked( GtkWidget *widget, gpointer   data )
{
	gint ret;

	BuildArguments();
	ret=fgk_Launch(cArgumentString,"");
	if(ret!=0) {
		error_message(window,"Failed executing fgfs");
	}
}	

static void but_save_clicked( GtkWidget *widget, gpointer   data )
{
	if(fgk_PutSettings(&current,OPTFILE)<0) {
		error_message(window, "Error saving settings file: "OPTFILE);
	}
}	

static void but_load_clicked( GtkWidget *widget, gpointer   data )
{
	LoadFile(OPTFILE);
}	

static void but_info_clicked( GtkWidget *widget, gpointer   data )
{
	gtk_message_about("FG Kicker, the flightgear launcher for Linux\n\nLicense: GPL\nProgram by\nChris De Maeyer <chrisdm@pandora.be>\nJoseph Harnish <bigjoe@linuxmail.org>\n\nNow go and roam the skies...\n","About FG Kicker");
}

static void key_sw_released( GtkWidget *widget, gpointer   data )
{
	gtk_message_about("FG Kicker, the flightgear launcher for Linux\n\nLicense: GPL\nProgram by\nChris De Maeyer <chrisdm@pandora.be>\nJoseph Harnish <bigjoe@linuxmail.org>\n\nNow go and roam the skies...\n","About FG Kicker");
}

static void tree_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	gchar *namanya;

	if (gtk_tree_selection_get_selected (selection, &model, &iter))
	{
		gtk_tree_model_get (model, &iter, ID_COLUMN, &namanya, -1);

		strcpy(current.airport, namanya);

		g_free (namanya);
	} else {
		strcpy(current.airport, "");

	}
}	

static void check_splash_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.splash = 1;
	} else {
		current.splash = 0;
	}
}

static void check_gamemode_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.gamemode = 1;
	} else {
		current.gamemode = 0;
	}
}

static void check_fuelmode_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.fuelmode = 1;
	} else {
		current.fuelmode = 0;
	}
}

static void check_intromusic_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.intromusic = 1;
	} else {
		current.intromusic = 0;
	}
}

static void check_mousepointer_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.mousepointer = 1;
	} else {
		current.mousepointer = 0;
	}
}

static void check_clockfreeze_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.clockfreeze = 1;
	} else {
		current.clockfreeze = 0;
	}
}

static void check_autocoord_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.autocoord = 1;
	} else {
		current.autocoord = 0;
	}
}

static void check_robjects_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.robjects = 1;
	} else {
		current.robjects = 0;
	}
}

static void check_freeze_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.freeze = 1;
	} else {
		current.freeze = 0;
	}
}

static void check_matchreal_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.matchreal = 1;
	} else {
		current.matchreal = 0;
	}
}

static void check_skyblend_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.skyblend = 1;
	} else {
		current.skyblend = 0;
	}
}

static void check_textures_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.textures = 1;
	} else {
		current.textures = 0;
	}
}

static void check_clouds_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.clouds = 1;
	} else {
		current.clouds = 0;
	}
}

static void check_units_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.units = 1;
	} else {
		current.units = 0;
	}
}

static void check_horizon_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.horizon = 1;
	} else {
		current.horizon = 0;
	}
}

static void check_lighting_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.lighting = 1;
	} else {
		current.lighting = 0;
	}
}

static void check_httpd_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.httpd = 1;
	} else {
		current.httpd = 0;
	}
}

static void tooltips_clicked (GtkWidget *widget, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		current.tooltips = 1;
		gtk_tooltips_disable(tips);
	} else {
		current.tooltips = 0;
		gtk_tooltips_enable(tips);

	}
}

static gboolean delete_event( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
	CleanExit();
	gtk_main_quit();

	return FALSE;
}

static void destroy( GtkWidget *widget, gpointer   data )
{
	CleanExit();
	gtk_main_quit ();
}


int main( int   argc, char *argv[] )
{
	GtkWidget *frame1;

	gchar title[64];

	Initialize();

	gtk_init (&argc, &argv);
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_add_events(window, GDK_KEY_RELEASE_MASK);

	tips =g_object_new(GTK_TYPE_TOOLTIPS,NULL);

	g_signal_connect (G_OBJECT (window), "delete_event",
		G_CALLBACK (delete_event), NULL);
	g_signal_connect (G_OBJECT (window), "destroy",
		G_CALLBACK (destroy), NULL);

	sprintf(title,"Flightgear Kicker! v"MYVERSION" [using GTK v%d.%d.%d]",gtk_major_version,gtk_minor_version,gtk_micro_version);
	gtk_window_set_title (GTK_WINDOW (window), title);
	gtk_container_set_border_width (GTK_CONTAINER (window), 10);

	vboxm = gtk_vbox_new (FALSE, 0);

	hboxm = gtk_hbox_new (FALSE, 0);

	note = gtk_notebook_new ();

	/* page "General" */
	vboxg0 = gtk_vbox_new (FALSE, 0);
	hboxg1 = gtk_hbox_new (FALSE, 0);
	vboxg10 = gtk_vbox_new (FALSE, 0);
	vboxg11 = gtk_vbox_new (FALSE, 0);
	hboxg2 = gtk_hbox_new (FALSE, 0);

	gtk_container_add (GTK_CONTAINER (hboxg1), vboxg10);
	gtk_container_add (GTK_CONTAINER (hboxg1), vboxg11);	

	frameg1 = gtk_frame_new (" Toggles ");
	gtk_container_set_border_width (GTK_CONTAINER (frameg1), 10);
	frameg2 = gtk_frame_new (" Selection ");
	gtk_container_set_border_width (GTK_CONTAINER (frameg2), 10);

	gtk_container_add (GTK_CONTAINER (frameg1), hboxg1);
	gtk_container_add (GTK_CONTAINER (vboxg0), frameg1);
	labelg = gtk_label_new (" General ");
	gtk_container_add (GTK_CONTAINER (frameg2), hboxg2);
	gtk_container_add (GTK_CONTAINER (vboxg0), frameg2);

	/* check_buttons */
	check_splash = gtk_check_button_new_with_label("Splash screen");
	g_signal_connect (G_OBJECT (check_splash), "clicked",
		G_CALLBACK (check_splash_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg10), check_splash, TRUE, TRUE, 0);

	check_gamemode = gtk_check_button_new_with_label("Game mode");
	g_signal_connect (G_OBJECT (check_gamemode), "clicked",
		G_CALLBACK (check_gamemode_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg10), check_gamemode, TRUE, TRUE, 0);

	check_fuelmode = gtk_check_button_new_with_label("Fuel freeze");
	g_signal_connect (G_OBJECT (check_fuelmode), "clicked",
		G_CALLBACK (check_fuelmode_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg10), check_fuelmode, TRUE, TRUE, 0);

	check_intromusic = gtk_check_button_new_with_label("Intro music");
	g_signal_connect (G_OBJECT (check_intromusic), "clicked",
		G_CALLBACK (check_intromusic_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg10), check_intromusic, TRUE, TRUE, 0);

	check_mousepointer = gtk_check_button_new_with_label("Mouse pointer");
	g_signal_connect (G_OBJECT (check_mousepointer), "clicked",
		G_CALLBACK (check_mousepointer_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg11), check_mousepointer, TRUE, TRUE, 0);

	check_autocoord = gtk_check_button_new_with_label("Auto coordination");
	g_signal_connect (G_OBJECT (check_autocoord), "clicked",
		G_CALLBACK (check_autocoord_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg11), check_autocoord, TRUE, TRUE, 0);

	check_freeze = gtk_check_button_new_with_label("Freeze");
	g_signal_connect (G_OBJECT (check_freeze), "clicked",
		G_CALLBACK (check_freeze_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg11), check_freeze, TRUE, TRUE, 0);

	check_units = gtk_check_button_new_with_label("Units meters");
	g_signal_connect (G_OBJECT (check_units), "clicked",
		G_CALLBACK (check_units_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg11), check_units, TRUE, TRUE, 0);

	check_httpd = gtk_check_button_new_with_label("Enable Http Host");
	g_signal_connect (G_OBJECT (check_httpd), "clicked",
		G_CALLBACK (check_httpd_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxg11), check_httpd, TRUE, TRUE, 0);

	/* aircraft combo */
	label_aircraft = gtk_label_new("Aircraft");
	gtk_widget_show (label_aircraft);
	combo_aircraft = gtk_combo_new();
	FillAircrafts();
	gtk_combo_set_popdown_strings (GTK_COMBO (combo_aircraft), combo_aircraft_list);
	gtk_box_pack_start (GTK_BOX (hboxg2), label_aircraft, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (hboxg2), combo_aircraft, TRUE, TRUE, 0);

	gtk_widget_set_size_request(hboxg1,640,120);
	gtk_widget_set_size_request(hboxg2,640,48);

	/* page "Render" */
	vboxr0 = gtk_vbox_new (FALSE, 0);
	hboxr1 = gtk_hbox_new (FALSE, 0);
	vboxr10 = gtk_vbox_new (FALSE, 0);
	vboxr11 = gtk_vbox_new (FALSE, 0);

	gtk_container_add (GTK_CONTAINER (hboxr1), vboxr10);
	gtk_container_add (GTK_CONTAINER (hboxr1), vboxr11);	

	framer1 = gtk_frame_new (" Toggles ");
	gtk_container_set_border_width (GTK_CONTAINER (framer1), 10);

	gtk_container_add (GTK_CONTAINER (framer1), hboxr1);
	gtk_container_add (GTK_CONTAINER (vboxr0), framer1);
	labelr = gtk_label_new (" Render ");

	check_robjects = gtk_check_button_new_with_label("Random objects");
	g_signal_connect (G_OBJECT (check_robjects), "clicked",
		G_CALLBACK (check_robjects_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxr10), check_robjects, TRUE, TRUE, 0);

	check_skyblend = gtk_check_button_new_with_label("Sky blend");
	g_signal_connect (G_OBJECT (check_skyblend), "clicked",
		G_CALLBACK (check_skyblend_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxr10), check_skyblend, TRUE, TRUE, 0);

	check_textures = gtk_check_button_new_with_label("Textures");
	g_signal_connect (G_OBJECT (check_textures), "clicked",
		G_CALLBACK (check_textures_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxr10), check_textures, TRUE, TRUE, 0);

	check_clouds = gtk_check_button_new_with_label("Clouds");
	g_signal_connect (G_OBJECT (check_clouds), "clicked",
		G_CALLBACK (check_clouds_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxr11), check_clouds, TRUE, TRUE, 0);

	check_horizon = gtk_check_button_new_with_label("Horizon effect");
	g_signal_connect (G_OBJECT (check_horizon), "clicked",
		G_CALLBACK (check_horizon_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxr11), check_horizon, TRUE, TRUE, 0);

	check_lighting = gtk_check_button_new_with_label("Enhanced lighting");
	g_signal_connect (G_OBJECT (check_lighting), "clicked",
		G_CALLBACK (check_lighting_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxr11), check_lighting, TRUE, TRUE, 0);

	gtk_widget_set_size_request(hboxr1,640,180);

	/* page "Time" */
	vboxt0 = gtk_vbox_new (FALSE, 0);
	hboxt1 = gtk_hbox_new (FALSE, 0);
	vboxt10 = gtk_vbox_new (FALSE, 0);
	vboxt11 = gtk_vbox_new (FALSE, 0);

	gtk_container_add (GTK_CONTAINER (hboxt1), vboxt10);
	gtk_container_add (GTK_CONTAINER (hboxt1), vboxt11);

	framet1 = gtk_frame_new (" Toggles ");
	gtk_container_set_border_width (GTK_CONTAINER (framet1), 10);

	gtk_container_add (GTK_CONTAINER (framet1), hboxt1);
	gtk_container_add (GTK_CONTAINER (vboxt0), framet1);
	labelt = gtk_label_new (" Time ");

	check_matchreal = gtk_check_button_new_with_label("Match real time");
	g_signal_connect (G_OBJECT (check_matchreal), "clicked",
		G_CALLBACK (check_matchreal_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxt10), check_matchreal, TRUE, TRUE, 0);

	check_clockfreeze = gtk_check_button_new_with_label("Clock freeze");
	g_signal_connect (G_OBJECT (check_clockfreeze), "clicked",
		G_CALLBACK (check_clockfreeze_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxt10), check_clockfreeze, TRUE, TRUE, 0);

	gtk_widget_set_size_request(hboxt1,640,180);

	/* page "FgKicker" */
	vboxf0 = gtk_vbox_new (FALSE, 0);
	hboxf1 = gtk_hbox_new (FALSE, 0);
	vboxf10 = gtk_vbox_new (FALSE, 0);
	vboxf11 = gtk_vbox_new (FALSE, 0);

	gtk_container_add (GTK_CONTAINER (hboxf1), vboxf10);
	gtk_container_add (GTK_CONTAINER (hboxf1), vboxf11);

	framef1 = gtk_frame_new (" Toggles ");
	gtk_container_set_border_width (GTK_CONTAINER (framef1), 10);

	gtk_container_add (GTK_CONTAINER (framef1), hboxf1);
	gtk_container_add (GTK_CONTAINER (vboxf0), framef1);
	labelf = gtk_label_new (" FgKicker ");

	tooltips= gtk_check_button_new_with_label("Disable Tooltips");
	g_signal_connect (G_OBJECT (tooltips), "clicked",
		G_CALLBACK (tooltips_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(vboxf10), tooltips, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tips,GTK_WIDGET(tooltips),"Disable FgKicker tooltips on mouse-over","");

	gtk_widget_set_size_request(hboxf1,640,180);

	/* page "Airport" */
	vboxa0 = gtk_vbox_new (FALSE, 0);
	vboxa1  = gtk_vbox_new (FALSE, 0);
	hboxa0  = gtk_hbox_new (FALSE,0);

	framea1 = gtk_frame_new (" Airport ");
	gtk_container_set_border_width (GTK_CONTAINER (framea1), 10);

	gtk_container_add (GTK_CONTAINER (vboxa1),hboxa0);
	gtk_container_add (GTK_CONTAINER (framea1),vboxa1);
	gtk_container_add (GTK_CONTAINER (vboxa0),framea1);
	labela = gtk_label_new (" Airport ");

	store = gtk_tree_store_new (N_COLUMNS,G_TYPE_STRING,G_TYPE_STRING);
	FillAirports(store);
	tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	g_object_unref (G_OBJECT (store));

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Location", renderer, "text", NAME_COLUMN,NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
	column = gtk_tree_view_column_new_with_attributes ("ID", renderer, "text", ID_COLUMN,NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

	gtk_tree_view_set_enable_search(GTK_TREE_VIEW (tree),TRUE); 
	gtk_tree_view_set_search_column(GTK_TREE_VIEW (tree),0);

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(hboxa0), scrolled_window, TRUE, TRUE, 0);

	gtk_container_add (GTK_CONTAINER (scrolled_window),tree);

	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree)); 
	gtk_tree_selection_set_mode (sel, GTK_SELECTION_SINGLE);
	g_signal_connect (G_OBJECT (sel), "changed",G_CALLBACK (tree_selection_changed_cb),NULL);
	g_signal_connect (G_OBJECT (scrolled_window), "key-release-event",G_CALLBACK (key_sw_released),NULL);

	gtk_widget_set_size_request(tree,600,120);
	gtk_widget_set_size_request(vboxa0,640,180);

	/* compose pages in book */
	gtk_notebook_append_page (GTK_NOTEBOOK (note), vboxg0, labelg);
	gtk_notebook_append_page (GTK_NOTEBOOK (note), vboxa0, labela);
	gtk_notebook_append_page (GTK_NOTEBOOK (note), vboxr0, labelr);
	gtk_notebook_append_page (GTK_NOTEBOOK (note), vboxt0, labelt);
	gtk_notebook_append_page (GTK_NOTEBOOK (note), vboxf0, labelf);

	/* command buttons */
	button_kick = gtk_button_new_with_label ("Kick!");
	g_signal_connect (G_OBJECT (button_kick), "clicked",
		G_CALLBACK (but_kick_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(hboxm), button_kick, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tips,GTK_WIDGET(button_kick),"Launch Flightgear with selected options","");

	button_load = gtk_button_new_with_label ("Reload Default");
	g_signal_connect (G_OBJECT (button_load), "clicked",
		G_CALLBACK (but_load_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(hboxm), button_load, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tips,GTK_WIDGET(button_load),"Reload options previously saved as default","");

	button_save = gtk_button_new_with_label ("Save as Default");
	g_signal_connect (G_OBJECT (button_save), "clicked",
		G_CALLBACK (but_save_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(hboxm), button_save, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tips,GTK_WIDGET(button_save),"Save current options as default (loaded on startup)","");

	button_info = gtk_button_new_with_label ("Info");
	g_signal_connect (G_OBJECT (button_info), "clicked",
		G_CALLBACK (but_info_clicked), NULL);
	gtk_box_pack_start (GTK_BOX(hboxm), button_info, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tips,GTK_WIDGET(button_info),"Some information about this program","");

	button_quit = gtk_button_new_with_label ("Quit");
	g_signal_connect (G_OBJECT (button_quit), "clicked",
		G_CALLBACK (but_quit_clicked), NULL);
	g_signal_connect_swapped (G_OBJECT (button_quit), "clicked",
		G_CALLBACK (gtk_widget_destroy),
		G_OBJECT (window));
	gtk_box_pack_start (GTK_BOX(hboxm), button_quit, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tips,GTK_WIDGET(button_quit),"Quit FgKicker or what else","");

	gtk_container_add (GTK_CONTAINER (vboxm), note);	
	gtk_container_add (GTK_CONTAINER (vboxm), hboxm);	
	gtk_container_add (GTK_CONTAINER (window), vboxm);

	gtk_widget_show_all(GTK_WIDGET(window));

	/* load defaults */
	LoadFile(OPTFILE);

	gtk_main ();
	return 0;
}
